package com.gree.mall.manager.logic.admin;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.gree.mall.manager.bean.admin.AdminUserCom;
import com.gree.mall.manager.bean.admin.AdminWebsitTree;
import com.gree.mall.manager.exception.RemoteServiceException;

import com.gree.mall.manager.logic.admin.ew.EnterpriseWechatWorkerLogic;
import com.gree.mall.manager.logic.common.CommonLogic;
import com.gree.mall.manager.logic.common.WorkWechatLogic;
import com.gree.mall.manager.plus.entity.AdminCompanyWechat;
import com.gree.mall.manager.plus.entity.AdminWebsit;
import com.gree.mall.manager.plus.entity.AdminZone;
import com.gree.mall.manager.plus.service.AdminWebsitService;
import com.gree.mall.manager.plus.service.AdminZoneService;
import com.gree.mall.manager.plus.service.EnterpriseWechatWorkerService;
import lombok.extern.slf4j.Slf4j;
import me.chanjar.weixin.common.error.WxErrorException;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Slf4j
@Service
public class AdminWebsitLogic {

    @Autowired
    WorkWechatLogic workWechatLogic;
    @Autowired
    AdminWebsitService adminWebsitService;
    @Autowired
    AdminZoneService adminZoneService;
    @Autowired
    CommonLogic commonLogic;
    @Autowired
    EnterpriseWechatWorkerLogic enterpriseWechatWorkerLogic;

    /**
     * 区域列表
     */
    public List<AdminZone> adminZoneList(){
        List<AdminZone> list = adminZoneService.list();
        return list;
    }

    /**
     * 部门树
     */
    public List<AdminWebsitTree> tree(HttpServletRequest request){
        AdminUserCom adminUser = commonLogic.getAdminUser(request);

        List<AdminWebsit> list = adminWebsitService.lambdaQuery()
                .in(CollectionUtils.isNotEmpty(adminUser.getCorpIds()),AdminWebsit::getCorpId,adminUser.getCorpIds())
                .list();
        List<AdminWebsitTree> trees = new ArrayList<>();
        for(AdminWebsit adminWebsit : list){
            AdminWebsitTree adminWebsitTree = new AdminWebsitTree();
            BeanUtils.copyProperties(adminWebsit,adminWebsitTree);
            trees.add(adminWebsitTree);
        }

        List<AdminWebsitTree> parentList = trees.stream().filter(x -> x.getParentId() == 0).collect(Collectors.toList());
        for(AdminWebsitTree adminWebsit:parentList){
            adminWebsit.setChildren(this.treeModule(trees,adminWebsit.getWebsitId(),adminWebsit.getCorpId()));
        }
        return parentList;
    }


    /**
     *  新增部门
     */
    @Transactional
    public void add(AdminWebsit adminWebsit, HttpServletRequest request) throws WxErrorException, RemoteServiceException {
        AdminUserCom adminUser = commonLogic.getAdminUser(request);

        Integer count = adminWebsitService.lambdaQuery()
                .eq(AdminWebsit::getWebsitNumber,adminWebsit.getWebsitNumber())
                .or()
                .eq(AdminWebsit::getName, adminWebsit.getName())
                .count();
        if(count > 0){
            throw new RemoteServiceException("部门名称或部门编号已存在");
        }
        if(adminWebsit.getParentId() == 0){
            throw new RemoteServiceException("请选择父部门");
        }

        //Long aLong = workWechatLogic.addOrganization(adminWebsit.getName(), adminWebsit.getParentId(),adminUser.getAdminCompanyWechat());
        Long aLong = IdWorker.getId();
        adminWebsit.setId(aLong+"");
        adminWebsit.setCorpId(adminUser.getCorpIds().get(0));
        adminWebsit.setWebsitId(aLong);
        adminWebsit.insert();

    }
    /**
     * 编辑部门
     */
    @Transactional
    public void update(AdminWebsit adminWebsit,HttpServletRequest request) throws RemoteServiceException, WxErrorException {

        //AdminUserCom adminUser = commonLogic.getAdminUser(request);

        Integer count = adminWebsitService.lambdaQuery()
                .ne(AdminWebsit::getId,adminWebsit.getId())
                .and(v -> v.eq(AdminWebsit::getWebsitNumber,adminWebsit.getWebsitNumber())
                        .or()
                        .eq(AdminWebsit::getName, adminWebsit.getName())
                )
                .count();
        if(count > 0){
            throw new RemoteServiceException("部门名称或部门编号已存在");
        }
        if(adminWebsit.getParentId() == 0){
            throw new RemoteServiceException("请选择父部门");
        }
        adminWebsit.updateById();
        //workWechatLogic.updateOrganization(adminWebsit.getWebsitId(),adminWebsit.getName(),adminWebsit.getParentId(),adminUser.getAdminCompanyWechat());

        //刷新成员部门名称
        enterpriseWechatWorkerLogic.reloadEnterpriseWechatName(adminWebsit,request);

    }

    /**
     * 删除部门
     */
    @Transactional
    public void delete(Long id,HttpServletRequest request) throws WxErrorException {
//s        AdminUserCom adminUser = commonLogic.getAdminUser(request);
        //递归删除
        List<AdminWebsit> list = new ArrayList<>();
        list.addAll(this.queryAllChild(list,id));
        List<Long> ids = list.stream().map(AdminWebsit::getWebsitId).collect(Collectors.toList());
        ids.add(id);
        if(ids.size() > 0) {
//            for(Long idd : ids){
//                workWechatLogic.deleteOrganization(idd,adminUser.getAdminCompanyWechat());
//            }
            adminWebsitService.lambdaUpdate().in(AdminWebsit::getWebsitId, ids).remove();
        }
    }

    /**
     * 详情
     * @param id
     */
    public AdminWebsit detail(String id){
        AdminWebsit adminWebsit = adminWebsitService.getById(id);
        return adminWebsit;
    }

    /**
     * 详情
     */
    public AdminWebsit detail(Long websitId,String corpId){
        AdminWebsit adminWebsit = adminWebsitService.lambdaQuery()
                .eq(AdminWebsit::getWebsitId,websitId)
                .eq(AdminWebsit::getCorpId,corpId)
                .one();
        return adminWebsit;
    }

    /**
     * 递归查询所有数据
     */
    public List<AdminWebsit> queryAllChild(List<AdminWebsit> list,Long id){
        if(id == null || id == 0){
            return list;
        }
        List<AdminWebsit> adminWebsits = adminWebsitService.lambdaQuery().eq(AdminWebsit::getParentId, id).list();
        if(adminWebsits.size() == 0){
            return list;
        }
        for(AdminWebsit adminWebsit:adminWebsits){
            this.queryAllChild(list,adminWebsit.getWebsitId());
        }
        list.addAll(adminWebsits);
        return list;
    }

    /**
     * 递归组装数据
     * @param list
     * @param parentId
     * @return
     */
    private List<AdminWebsitTree> treeModule(List<AdminWebsitTree> list, Long parentId,String corpId){
        //最父级资源树
        if(parentId == null || parentId == 0) {
            return null;
        }
        List<AdminWebsitTree> collect = list.stream()
                .filter(e -> e.getParentId().equals(parentId) && e.getCorpId().equals(corpId)).collect(Collectors.toList());
        if(collect.size() == 0){
            return null;
        }
        for(AdminWebsitTree bean : collect){
            bean.setChildren(this.treeModule(list, bean.getWebsitId(),bean.getCorpId()));
        }
        return collect;
    }





}
